local siloClient = require("silo.client")

local targetItem = ({ ... })[1]

siloClient("smelter", function(ctx)
  local messaging = ctx:require("silo.messaging")

  ctx:addTask(function()
    while ctx.server == -1 do sleep(0.05) end

    local furnaces = {}

    for _, name in pairs(peripheral.getNames()) do
      if string.match(name, "furnace") then
        table.insert(furnaces, name)
      end
    end

    print("Smelting " .. targetItem .. " using " .. #furnaces .. " furnaces...")

    local counter = 0
    local awaiting = 0

    while true do
      sleep(0.25)
      if awaiting == 0 then
        local index = (counter % #furnaces) + 1
        counter = counter + 1

        local furnaceName = furnaces[index]
        local furnace = peripheral.wrap(furnaceName)

        local items = furnace.list()

        local needsInsertion = items[1] == nil or items[1].count < 2
        local needsFuel = items[2] == nil or items[2].count < 2
        local needsExtraction = items[3] ~= nil

        if needsExtraction then
          awaiting = awaiting + 1
          messaging:sendDirect(ctx.server, {
            type = "storage.deposit_stack",
            from = furnaceName,
            fromSlot = 3
          }, function(...)
            awaiting = awaiting - 1
          end)
        end

        if needsFuel then
          awaiting = awaiting + 1
          messaging:sendDirect(ctx.server, {
            type = "storage.withdraw_stack",
            hash = "minecraft:coal@0",
            to = furnaceName,
            toSlot = 2,
            count = 4
          }, function(...)
            awaiting = awaiting - 1
          end)
        end

        if needsInsertion then
          awaiting = awaiting + 1
          messaging:sendDirect(ctx.server, {
            type = "storage.withdraw_stack",
            hash = targetItem,
            to = furnaceName,
            toSlot = 1,
            count = 8
          }, function(...)
            awaiting = awaiting - 1
          end)
        end
      end
    end
  end)

  ctx:run()
end)
